#include "MarkerCollection.h"


// ================================================================
// GenomeMixer    (c) 2003, Alex Williams
// All rights reserved.
// Send email correspondence to Robert W. Williams <rwilliam@nb.utmem.edu>
// at the University of Tennesse Health Science Center

// This project was funded jointly by the National Institute of Mental Health, National Institute on Drug Abuse,
// and the National Science Foundation (award P20-MH 62009 to KFM and RW)

// This source code may be used according to the terms of the GNU General Public License (GPL).
// See www.gnu.org for specifics. Essentially, the source may be freely used as long as the source code
// for whatever program the source is also made freely available. If you want to use this source
// in a closed-source program, you should email us for licensing options.

// Disclaimer: This code comes with no warranty at all, and there is no guarantee that it will
// work as claimed in the documentation. Hopefully, it will be useful anyway.

// Note that GenomeMixer makes use of the Trolltech Qt toolkit for its window-managing needs.
// There is very little use of Qt data structures (aside from QString) in the actual processing code,
// but **you will need to have Qt installed in order to compile this code.** Check <www.trolltech.com>
// for details; there should be a free GPL-software-only version of Qt available on the Trolltech
// web site.

// Any redistributions of this software in code or binary form must include the
// above copyright notice, licensing information, disclaimer, and this notice itself.
// Basically, all the comments at the top of this file must be retained.
// ================================================================


MarkerCollection::MarkerCollection(unsigned int initCapacity) { // Constructor
    myTotalMarkers = 0;
    data.reserve(initCapacity);
}

void MarkerCollection::clearAllMarkerData(void) {
  myTotalMarkers = 0;
  data.reserve(10);
  data.clear();
}

void MarkerCollection::addMarker(int chr, string name, long loc, float cM) {
    assert(chr >= 0 && loc >= 0 && cM >= 0);
    
    /*
    if (chr >= Chromosome::numChr()) {
	cout << "@@ >>> Warning: Added a marker on a currently-invalid chromosome.";	
    } else if (loc > Chromosome::lengthBasesFromIndex(chr) ||
	 cM > Chromosome::length_cM_fromIndex(chr)) {
        cout << "@@ >>> Warning: added a marker that is beyond the length of the chromosome: "
             << Chromosome::chrNameFromIndex(chr)
             << " (index " << chr << "): \""
             << name << "\" (bases: " << loc << ", max is "
             << Chromosome::lengthBasesFromIndex(chr)
             << ") (cM: " << cM << ", max is "
             << Chromosome::length_cM_fromIndex(chr)
             << ")" << endl;
        
    }*/
    
    if (chr > 100) {
	cerr << "You have a marker on chromosome " << chr << ", which is too great a number for us to deal with at the moment.";
	exit(1);
    }
    
    while (chr >= long(data.size())) {
	// For each chromosome, we have to add a vector ("theVec"). This vector then stores all the marker data for that particular chromosome.
	std::vector<ChrDataStruct> theVec; // <-- permanent? temporary?
	data.push_back(theVec);
    }
    
    ChrDataStruct mark;
    mark.location = loc;
    mark.name     = name;
    mark.cM       = cM;
	
    assert(chr >= 0 && chr < long(data.size()));
    data[chr].push_back(mark);
    
    myTotalMarkers++;	// update the total marker count
}


string MarkerCollection::name(int chrIndex, long index) {
    assert( chrIndex >= 0 && chrIndex < long(data.size()) );
    assert( index >= 0 && index < long((data[chrIndex]).size()) );
    return (data[chrIndex][index]).name;
}


int MarkerCollection::base(int chrIndex, long index) {
    assert( chrIndex >= 0 && chrIndex < long(data.size()) );
    assert( index >= 0 && index < long((data[chrIndex]).size()) );
    return (data[chrIndex][index]).location;
}


float MarkerCollection::cM(int chrIndex, long index) {
    assert( chrIndex >= 0 && chrIndex < long(data.size()) );
    assert( index >= 0 && index < long((data[chrIndex]).size()) );
    return (data[chrIndex][index]).cM;
}


/*double MarkerCollection::selection_ref_fromIndex(int chrIndex, long index) {
//    assert( chrIndex >= 0 && chrIndex < long(data.size()) );
//    assert( index >= 0 && index < long((data[chrIndex]).size()) );
//    return ((data[chrIndex]).selection[index]);
//    return \($self->{data}[$chrIndex][$index][SELECTION]);
//}
*/


int MarkerCollection::markersForHowManyChr() {
  int hasMarkersCounter = 0;
  for (unsigned int i = 0; i < data.size(); i++) {
    if (data[i].size() > 0) {
      hasMarkersCounter++; // If this chromosome actually has markers, then we count it.
    }
    // Otherwise, we don't count it.
  }
  return hasMarkersCounter;
}


int MarkerCollection::numberOfChrs(void) {
  return data.size();  // Accessor
}


long MarkerCollection::totalMarkers(void) {
  return myTotalMarkers; // Accessor
}


long MarkerCollection::markersInChr(int chrIndex) {
  assert(chrIndex >= 0 && chrIndex < long(data.size())); // <-- respect array bounds
  return data[chrIndex].size();
}

long MarkerCollection::mostMarkersOnAnyChr() {
    long mostSoFar = 0;
    for (int i = 0; i < numberOfChrs(); i++) {
	if (markersInChr(i) > mostSoFar) {
	    mostSoFar = markersInChr(i);
	}
    }
    return mostSoFar;
}

void MarkerCollection::printMarkerInfo() {
    cout << "Printing marker collection info:\n";
    for (int chr = 0; chr < numberOfChrs(); chr++) {
        for (long item = 0; item < markersInChr(chr); item++) {
	    
	    string nameOfChr = "(no name)";
	    float lengthOfChrCM = -1;
	    
	    if (chr < Chromosome::numChr()) {
		nameOfChr = Chromosome::chrNameFromIndex(chr);
		lengthOfChrCM = Chromosome::length_cM_fromIndex(chr);
	    }
	    
	    cout << "  Marker \"" << name(chr, item) << "\" on " << nameOfChr << " at base position " << base(chr, item) << " (" << cM(chr, item) << " cM";
	    
	    if (cM(chr, item) == 0)
		cout << ": start)" << endl;
	    else if (cM(chr, item) ==  lengthOfChrCM)
		cout << ": end)" << endl;
	    else
		cout << ")" << endl;
	    
        }
    }
    cout << "Done printing marker collection info.\n";
}



